//-----------------------------------------------------------------------
//
//  Evaluation of MLE Procedure
//
//                  
//          
//-----------------------------------------------------------------------
new;
library pgraph;
graphset; 



#include "repfiles_final\\EstimationProcs.ind";
#include "repfiles_final\\varmaAUXprocsV2.ind";
#include "repfiles_final\\VARestimation.ind";
#include "repfiles_final\\paraval.ind";

folder="repfiles_final\\";
outputdir=folder$+"output\\"; 

#include sqpSolveMT.sdf

//-----------------------------------------------------------------------
// Set simulation parameters,...:
expp=40; 



for jj (3,5,1); 
   
    exper=expp+jj;
    methode="MLE";
    TT=100;
    NN=1000;

    save_results=1; // 0 - no save 1 - save


//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// Load process data: 
    {KK,p_p,q_p,AA0_p,AA1_p,AA2_p,MM1_p,MM2_p,covar_p}=paravalues(exper);

// Load data 
    string1=folder$+"ndata\\ser";
    string1=string1$+ftocv(exper,1,0)$+"T"$+ftocv(TT,1,0)$+".txt";
    load ymat[(KK*NN),TT+4] = ^string1;


// Get parameters:
    ar_mat0=AA0_p;
    ar_mat=AA1_p;
    if p_p>1;
        ar_mat=ar_mat~AA2_p;
    endif;
    ma_mat=MM1_p;
    if q_p>1;
        ma_mat=ma_mat~MM2_p;
    endif;
     

    {RR,r,CCe,ce}=get_restrictions(exper);
 
// ... true parameter vector
    beta0=vec((eye(KK)-ar_mat0)~ar_mat~ma_mat);
    gamma0=inv(RR'RR)*RR'beta0;      
    numb_para=rows(gamma0);    

// ... true impulse repsonses
    irtrue=varma_irfun(ar_mat0,ar_mat,ma_mat,4);
    ir1_0=irtrue[.,2];
    ir4_0=irtrue[.,5];

    {dum_ir1,dum_ir4}=getMeasures(exper);


   

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// MONTE CARLO SIMULATION: 


    gmse_mat=0;
    fmse1_mat=0;
    fmse4_mat=0;
    irmse1_mat=0;
    irmse4_mat=0;

    gerr_mat={}; 
    ferr1_mat={}; 
    ferr4_mat={}; 
    irerr1_mat={}; 
    irerr4_mat={}; 
    
    statistic=0|0|0;
    frcst_mat={}; 
    gamma_mat={}; 
    gerror_mat={}; 

   

    for i (1,NN,1);
        //cls;i;;
  
        // Get series and de-mean it -------------------------- 

        y=ymat[((i-1)*KK+1):i*KK,.];
        y_eff=y[.,1:TT];
        meanY=meanc(y_eff');
        y_eff=y_eff-meanY;
                
        //-----------------------------------------------------          
        // Estimate Parameters:

        // starting values:
        para_v=vech(covar_p)|gamma0; 

        //para_v[5]=0.4; 

        // settings:                      
        struct PV par1;
        par1=pvCreate;
        par1=pvPack(par1,para_v,"parameters");
        
        struct DS d0;
        d0=dsCreate;
        d0.dataMatrix=y_eff'; 
        
        struct sqpSolveMTControl c0;
        c0=sqpSolveMTControlCreate;
        c0.maxIters=40; 

        // optimization:  
        struct sqpSolveMTout out1;
        out1 = sqpSolveMT(&varma_MLEloglik,par1,d0,c0);

        // get parameters: 
        pp=pvUnPack(out1.par,"parameters");
        
        n_vc=(KK*(KK+1)/2); 
        vc_res=xpnd(pp[1:n_vc,.]); 
        gamma_e=pp[n_vc+1:rows(pp),.]; 
       
        // compute residuals of invertible representation: 
        if meanc(abs(gamma_e))<1000;        
            
            {gamma_e,vc_res}=checkMApoly(gamma_e,vc_res,RR,r,KK,p_p,q_p); 

            struct PV par2;
            par2=pvCreate; 
            para_v2=vech(vc_res)|gamma_e; 
            par2=pvPack(par2,para_v2,"parameters"); 
            res_varma=getresvarma(par2,d0); 
 
        else; 
            res_varma={};         
        endif; 
        
        {AA0_e,AA_e,MM_e}=getvarma_mat(gamma_e,RR,r,KK,p_p,q_p); 

        out_code=out1.retcode; 

        // if optimization failed
        nTa=round(0.5*(TT)^(1/2));
        if ((out_code ne 0) and (out_code ne 2)) or scalerr(res_varma);
            {gamma_e,res_cov,res_varma}=varma_HR(y_eff,p_p,q_p,RR,r,nTa,1,0);
            {AA0_e,AA_e,MM_e}=getvarma_mat(gamma_e,RR,r,KK,p_p,q_p); 
        endif; 


        
       //------------------------------------------------------
        // Caculate MSE of parameter estimate: 
        g_error=gamma_e-gamma0;

        meig=check_maxeig(AA0_e,AA_e);

        outl=(meanc(abs(g_error))>2); 
        if meig>1;
            statistic[1]=statistic[1]+1;
        endif; 
        if outl==1;
            statistic[2]=statistic[2]+1; 
            AA_e; 
            MM_E;
        endif; 

 
        // if algorithm failed, use HR 
        if meig>1 or outl==1;   

           do while (meig>1 or outl==1); 
                
                {gamma_e,res_cov,res_varma}=varma_HR(y_eff,p_p,q_p,RR,r,nTa,1,1);
                {AA0_e,AA_e,MM_e}=getvarma_mat(gamma_e,RR,r,KK,p_p,q_p); 
                
                meig=check_maxeig(AA0_e,AA_e);
                g_error=gamma_e-gamma0;
                outl=(meanc(abs(g_error))>2); 
                
                nTa=nTa+1; 
            
                if nTa>round((TT)^(1/2));
                    {gamma_e,res_cov,res_varma}=varma_HR(y_eff,p_p,q_p,RR,r,round(0.5*(TT)^(1/2)),1,0);
                    {AA0_e,AA_e,MM_e}=getvarma_mat(gamma_e,RR,r,KK,p_p,q_p); 
                    g_error=gamma_e-gamma0;
                    meig=0.9; 
                    outl=0;                     
                endif; 

           endo; 

        endif; 
            
        gmse_mat=gmse_mat+g_error*(g_error');


        //------------------------------------------------------
        // FORECASTS: 
        {yfc_mat}=varma_frc(y_eff,res_varma,AA0_e,AA_e,MM_e,4); // Checked for pure VARs.
        f_error1=(meanY+yfc_mat[.,1])-y[.,TT+1];
        f_error4=(meanY+yfc_mat[.,4])-y[.,TT+4];
        
       
        fmse1_mat=fmse1_mat+f_error1*(f_error1');
        fmse4_mat=fmse4_mat+f_error4*(f_error4');

        //--------------------------------------------------------
        // IMPULSE RESPONSES: 
        irest=varma_irfun(AA0_e,AA_e,MM_e,4);
   
        ir_error1=delif(irest[.,2],dum_ir1)-delif(ir1_0,dum_ir1);
        ir_error4=delif(irest[.,5],dum_ir4)-delif(ir4_0,dum_ir4);

        irmse1_mat=irmse1_mat+ir_error1*(ir_error1');
        irmse4_mat=irmse4_mat+ir_error4*(ir_error4');


          // Save series of errors: 
        gamma_mat=gamma_mat~gamma_e; 
        gerror_mat=gerror_mat~g_error; 
        ferr1_mat=ferr1_mat~f_error1; 
        ferr4_mat=ferr4_mat~f_error4; 
        irerr1_mat=irerr1_mat~ir_error1; 
        irerr4_mat=irerr4_mat~ir_error4; 

    endfor;


    // Getting Means
    gmse_mat=gmse_mat/NN;
    fmse1_mat=fmse1_mat/NN;
    fmse4_mat=fmse4_mat/NN;
    irmse1_mat=irmse1_mat/NN;
    irmse4_mat=irmse4_mat/NN;


    //xy(seqa(1,1,NN),gerror_mat'); 





//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// PRINT:

format 3,3; 

print "--------------------------------------"; 
methode; 
print "experiment:" exper;
print "--------------------------------------"; 
print "Parameters:" sumc(diag(gmse_mat)); //det(gmse_mat);
print "Impulse Responses:"; 
print "h=1: " sumc(diag(irmse1_mat));//det(irmse1_mat);
print "h=4: " sumc(diag(irmse4_mat));//det(irmse4_mat); 

print "Forecasts:";
print "h=1: " sumc(diag(fmse1_mat));//det(fmse1_mat);
print "h=4: " sumc(diag(fmse4_mat));//det(fmse4_mat); 

print "Outliers:"
statistic;

gamma_m=meanc(gamma_mat');
getvarma_mat(gamma_m,RR,r,KK,p_p,q_p);




//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// SAVING:

    if save_results==1;
            
            stringA=outputdir$+"\\gmse"$+methode$+"_ser";
            stringA=stringA$+ftocv(exper,1,0)$+"T"$+ftocv(TT,1,0);
            save ^stringA=gmse_mat;

            stringB=outputdir$+"\\fmse_h1_"$+methode$+"_ser";
            stringB=stringB$+ftocv(exper,1,0)$+"T"$+ftocv(TT,1,0);
            save ^stringB=fmse1_mat;

            stringB=outputdir$+"\\fmse_h4_"$+methode$+"_ser";
            stringB=stringB$+ftocv(exper,1,0)$+"T"$+ftocv(TT,1,0);
            save ^stringB=fmse4_mat;
            
            stringC=outputdir$+"\\irmse_h1_"$+methode$+"_ser";
            stringC=stringC$+ftocv(exper,1,0)$+"T"$+ftocv(TT,1,0);
            save ^stringC=irmse1_mat;

            stringC=outputdir$+"\\irmse_h4_"$+methode$+"_ser";
            stringC=stringC$+ftocv(exper,1,0)$+"T"$+ftocv(TT,1,0);
            save ^stringC=irmse4_mat;

            stringD=outputdir$+"\\stat"$+methode$+"_ser";
            stringD=stringD$+ftocv(exper,1,0)$+"T"$+ftocv(TT,1,0);
            save ^stringD=statistic;
            
    endif;


endfor; 

        
     
      proc(1)=varma_MLEloglik(struct PV par1, struct DS d0);
        local n_vc,para_v,y,vc,gamma_v,AA0,AA,MM,phi,theta;
        local loglik,oldval,Ae,mroot;  
        
            n_vc=(KK*(KK+1)/2); 
            
            para_v=pvUnpack(par1,"parameters");
               
            y=d0.dataMatrix;
                 
            vc=xpnd(para_v[1:n_vc,.]); 
            gamma_v=para_v[n_vc+1:rows(para_v),.]; 

//          {gamma_v,vc}=checkMApoly(gamma_v,vc,RR,r,KK,p_p,q_p);
//          {AA0,AA,MM}=getvarma_mat(gamma_v,RR,r,KK,p_p,q_p); 
          

          
            oldval=trapchk(1);
            
            trap 1,1;
                Ae = invpd(vc); //inv(AA0);         mroot=check_minroot(AA0,MM); 
            trap oldval,1;


            if scalerr(Ae);// or abs(mroot)<1;
                loglik=-10000; 
            else;
 
                {gamma_v,vc}=checkMApoly(gamma_v,vc,RR,r,KK,p_p,q_p);
                {AA0,AA,MM}=getvarma_mat(gamma_v,RR,r,KK,p_p,q_p); 
    
    
                phi=inv(AA0)*AA[.,1:KK];
                if p_p>1; 
                    for p_i (2,p_p,1); 
                        phi=phi|(inv(AA0)*AA[.,(KK*(p_i-1)+1):(KK*p_i)]);
                    endfor; 
                endif; 
                
                theta=inv(AA0)*MM[.,1:KK]; 
                if q_p>1; 
                    for q_i (2,q_p,1); 
                        theta=theta|(inv(AA0)*MM[.,(KK*(q_i-1)+1):(KK*q_i)]);
                    endfor; 
                endif; 
             
      
                loglik=varmall(y,phi,-theta,vc); 


            endif;


            retp(-loglik); 
        endp; 
        
        
        
        
        
        proc(1)=getresvarma(struct PV par1, struct DS d0);
        local n_vc,para_v,y,vc,gamma_v,AA0,AA,MM,phi,theta;
        local resvarma,oldval,Ae;  
        
            n_vc=(KK*(KK+1)/2); 
            
            para_v=pvUnpack(par1,"parameters");
         
            y=d0.dataMatrix;
                         
            vc=xpnd(para_v[1:n_vc,.]); 
            gamma_v=para_v[n_vc+1:rows(para_v),.]; 

 
            {gamma_v,vc}=checkMApoly(gamma_v,vc,RR,r,KK,p_p,q_p);
            {AA0,AA,MM}=getvarma_mat(gamma_v,RR,r,KK,p_p,q_p); 
            
         
            phi=inv(AA0)*AA[.,1:KK];
            if p_p>1; 
                for p_i (2,p_p,1); 
                    phi=phi|(inv(AA0)*AA[.,(KK*(p_i-1)+1):(KK*p_i)]);
                endfor; 
            endif; 
            
            theta=inv(AA0)*MM[.,1:KK]; 
            if q_p>1; 
                for q_i (2,q_p,1); 
                    theta=theta|(inv(AA0)*MM[.,(KK*(q_i-1)+1):(KK*q_i)]);
                endfor; 
            endif; 
         
            resvarma=varmares(y,phi,-theta); 
   

            retp(resvarma'); 
        endp; 

