//-----------------------------------------------------------------------
//
//  Evaluation of VAR(AIC)
//
//                  
//          
//-----------------------------------------------------------------------
new;
library pgraph;
graphset; 




#include "repfiles_final\\EstimationProcs.ind";
#include "repfiles_final\\varmaAUXprocsV2.ind";
#include "repfiles_final\\VARestimation.ind";
#include "repfiles_final\\paraval.ind";


folder="repfiles_final\\";
outputdir=folder$+"output\\"; 

//-----------------------------------------------------------------------
// Set simulation parameters,...:

expp=40; 

for jj (1,5,1);   

    exper=expp+jj;
    methode="VAR(AIC)";
    TT=100;
    NN=1000;

    save_results=1; // 0 - no save 1 - save


//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// Load process data: 
    {KK,p_p,q_p,AA0_p,AA1_p,AA2_p,MM1_p,MM2_p,covar_p}=paravalues(exper);

// Load data 
    string1=folder$+"ndata\\ser";
    string1=string1$+ftocv(exper,1,0)$+"T"$+ftocv(TT,1,0)$+".txt";
    load ymat[(KK*NN),TT+4] = ^string1;


// Get parameters:
    ar_mat0=AA0_p;
    ar_mat=AA1_p;
    if p_p>1;
        ar_mat=ar_mat~AA2_p;
    endif;
    ma_mat=MM1_p;
    if q_p>1;
        ma_mat=ma_mat~MM2_p;
    endif;
     

    {RR,r,CCe,ce}=get_restrictions(exper);
 
// ... true parameter vector
    beta0=vec((eye(KK)-ar_mat0)~ar_mat~ma_mat);
    gamma0=inv(RR'RR)*RR'beta0;      
    numb_para=rows(gamma0);    

// ... true impulse repsonses
    irtrue=varma_irfun(ar_mat0,ar_mat,ma_mat,4);
    ir1_0=irtrue[.,2];
    ir4_0=irtrue[.,5];

    {dum_ir1,dum_ir4}=getMeasures(exper);


   

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// MONTE CARLO SIMULATION: 



    fmse1_mat=0;
    fmse4_mat=0;
    irmse1_mat=0;
    irmse4_mat=0;

    ferr1_mat={}; 
    ferr4_mat={}; 
    irerr1_mat={}; 
    irerr4_mat={}; 
    
    statistic=0|0|0;

   

    for i (1,NN,1);
        //cls;i;;
        // Get series and de-mean it -------------------------- 

        y=ymat[((i-1)*KK+1):i*KK,.];
        y_eff=y[.,1:TT];
        meanY=meanc(y_eff');
        y_eff=y_eff-meanY;
                
        //-----------------------------------------------------          
        // Estimate Parameters:

        method=1; //ols
        mud=0; //no intercept
        maxp=round(0.5*(TT)^(1/2));
        {aic,aic_p,hq,hq_p,sc,sc_p}=ord_VAR(y_eff,mud,1,maxp,method);


        {AA_e,res_cov,res_var}=var_ols(y_eff,mud,aic_p);
        AA0_e=eye(KK); 
        MM_e=zeros(KK,KK); 





       //------------------------------------------------------
        // Check parameter estimate: 

        if check_maxeig(AA0_e,AA_e)>1;
            {AA_e,res_cov,res_var}=var_yw(y_eff,aic_p);
            AA0_e=eye(KK); 
            MM_e=zeros(KK,KK); 
            statistic[1]=statistic[1]+1; 
        endif; 



        //------------------------------------------------------
        // FORECASTS: 
        {yfc_mat}=varma_frc(y_eff,res_var,AA0_e,AA_e,MM_e,4); // Checked for pure VARs.
        f_error1=(meanY+yfc_mat[.,1])-y[.,TT+1];
        f_error4=(meanY+yfc_mat[.,4])-y[.,TT+4];
      
        fmse1_mat=fmse1_mat+f_error1*(f_error1');
        fmse4_mat=fmse4_mat+f_error4*(f_error4');

        //--------------------------------------------------------
        // IMPULSE RESPONSES: 
        irest=varma_irfun(AA0_e,AA_e,MM_e,4);
   
        ir_error1=delif(irest[.,2],dum_ir1)-delif(ir1_0,dum_ir1);
        ir_error4=delif(irest[.,5],dum_ir4)-delif(ir4_0,dum_ir4);

        irmse1_mat=irmse1_mat+ir_error1*(ir_error1');
        irmse4_mat=irmse4_mat+ir_error4*(ir_error4');


        // Save series of errors: 

        ferr1_mat=ferr1_mat~f_error1; 
        ferr4_mat=ferr4_mat~f_error4; 
        irerr1_mat=irerr1_mat~ir_error1; 
        irerr4_mat=irerr4_mat~ir_error4; 

    endfor;


    // Getting Means

    fmse1_mat=fmse1_mat/NN;
    fmse4_mat=fmse4_mat/NN;
    irmse1_mat=irmse1_mat/NN;
    irmse4_mat=irmse4_mat/NN;





//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// PRINT:

format 3,3; 

print "--------------------------------------"; 
methode;
print "experiment:" exper;
print "--------------------------------------"; 
print "Impulse Responses:"; 
print "h=1: " det(irmse1_mat);
print "h=4: " det(irmse4_mat); 

print "Forecasts:";
print "h=1: " det(fmse1_mat);
print "h=4: " det(fmse4_mat); 

print "Outliers:"
statistic;




//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// SAVING:

    if save_results==1;
            

            stringB=outputdir$+"\\fmse_h1_"$+methode$+"_ser";
            stringB=stringB$+ftocv(exper,1,0)$+"T"$+ftocv(TT,1,0);
            save ^stringB=fmse1_mat;

            stringB=outputdir$+"\\fmse_h4_"$+methode$+"_ser";
            stringB=stringB$+ftocv(exper,1,0)$+"T"$+ftocv(TT,1,0);
            save ^stringB=fmse4_mat;
            
            stringC=outputdir$+"\\irmse_h1_"$+methode$+"_ser";
            stringC=stringC$+ftocv(exper,1,0)$+"T"$+ftocv(TT,1,0);
            save ^stringC=irmse1_mat;

            stringC=outputdir$+"\\irmse_h4_"$+methode$+"_ser";
            stringC=stringC$+ftocv(exper,1,0)$+"T"$+ftocv(TT,1,0);
            save ^stringC=irmse4_mat;

            stringD=outputdir$+"\\stat"$+methode$+"_ser";
            stringD=stringD$+ftocv(exper,1,0)$+"T"$+ftocv(TT,1,0);
            save ^stringD=statistic;
            
    endif;


endfor; 
